from dotenv import load_dotenv

from API.exchange_rates_service import ExchangeRatesService
from static import *

from borb.io.read.types import Decimal
from borb.pdf import Document, FixedColumnWidthTable, Page, PDF, Image
from borb.pdf import Paragraph, Alignment
from borb.pdf.canvas.font.simple_font.true_type_font import TrueTypeFont
from borb.pdf.canvas.geometry.rectangle import Rectangle
import pathlib
import datetime
import pandas as pd

load_dotenv()

PDF_OUTPUT = "data/file.pdf"


def convert_to_pdf_and_save_(products: pd.DataFrame, customer_data) -> None:
    final_price = get_final_price(products)

    # commercial offer building
    doc: Document = Document()
    page: Page = Page()
    doc.add_page(page)

    fonts = dict()
    fonts["regular_font"] = TrueTypeFont.true_type_font_from_file(
        pathlib.Path("static/fonts/aptos.ttf")
    )
    fonts["bold_font"] = TrueTypeFont.true_type_font_from_file(
        pathlib.Path("static/fonts/aptos-bold.ttf")
    )
    fonts["italic_font"] = TrueTypeFont.true_type_font_from_file(
        pathlib.Path("static/fonts/aptos-italic.ttf")
    )
    fonts["bold_italic_font"] = TrueTypeFont.true_type_font_from_file(
        pathlib.Path("static/fonts/aptos-extrabold-italic_2.ttf")
    )
    fonts["semibold_font"] = TrueTypeFont.true_type_font_from_file(
        pathlib.Path("static/fonts/aptos-semibold.ttf")
    )

    add_pre_info_(page, fonts, customer_data)

    pointer = {"lower_x": 50, "lower_y": 550}

    add_table_header_(page, fonts, pointer)

    # print("in convert_to_pdf_and_save_: " + str(len(products)))

    products = build_table_(page, products.astype(str), fonts, pointer)

    # that is for CO than more than 1 page...
    while len(products) != 0:
        page: Page = Page()
        doc.add_page(page)
        pointer["lower_y"] = 800
        products = build_table_(page, products.astype(str), fonts, pointer)

    if pointer["lower_y"] < 200:
        page: Page = Page()
        doc.add_page(page)
        pointer["lower_y"] = 800

    add_table_post_info_(page, fonts, final_price, pointer)

    with open(PDF_OUTPUT, "wb") as pdf_file_handle:
        PDF.dumps(pdf_file_handle, doc)


def add_pre_info_(page: Page, fonts: dict, customer_data) -> None:
    phone = customer_data["phone"]
    company_name = Paragraph(
        str("TOO «АСБ-ТОП»"),
        font=fonts["bold_font"],
        font_size=Decimal(16),
        horizontal_alignment=Alignment.CENTERED,
    )
    company_info = Paragraph(
        str(
            '''ПРОДАВЕЦ:
    ТОО «АСБ-ТОП» Республика Казахстан,
    010000, г. Астана, Алматинский район,
    ул. Жанкент, д 155, офис 6.
    Тел: +7 777 5336031
    РНН 620200517080
    Р/сч KZ71998BTB0000323564 KZT;
    БИК TSESKZKA;
    БИН 160540005119
    АО "Jýsan Bank"'''
        ),
        font=fonts["regular_font"],
        font_size=Decimal(8),
        respect_newlines_in_text=True,
    )
    header = Paragraph(
        str(
            f"Коммерческое предложение № 137 от {datetime.datetime.now().strftime('%d.%m.%Y')}"
        ),
        font=fonts["bold_italic_font"],
        font_size=Decimal(13),
        horizontal_alignment=Alignment.CENTERED,
    )
    recipient_info = Paragraph(
        str(f"ПОЛУЧАТЕЛЬ:\nКому, тел.:       {phone}"),
        font=fonts["regular_font"],
        font_size=Decimal(8),
        respect_spaces_in_text=True,
        respect_newlines_in_text=True,
    )
    success_pdf_img_path = pathlib.Path("static/success_pdf_image.jpg")
    success_pdf_image = Image(
        success_pdf_img_path, width=Decimal(64), height=Decimal(64)
    )
    pdf_delimiter_path = pathlib.Path("static/pdf_delimiter.png")
    pdf_delimiter = Image(
        pdf_delimiter_path, width=Decimal(500), height=Decimal(20)
    )
    company_name.paint(
        page, Rectangle(Decimal(200), Decimal(730), Decimal(200), Decimal(40))
    )
    success_pdf_image.paint(
        page, Rectangle(Decimal(450), Decimal(730), Decimal(80), Decimal(80))
    )
    pdf_delimiter.paint(
        page, Rectangle(Decimal(50), Decimal(730), Decimal(500), Decimal(20))
    )
    header.paint(
        page, Rectangle(Decimal(70), Decimal(685), Decimal(450), Decimal(30))
    )
    company_info.paint(
        page, Rectangle(Decimal(80), Decimal(550), Decimal(200), Decimal(130))
    )
    recipient_info.paint(
        page, Rectangle(Decimal(340), Decimal(550), Decimal(150), Decimal(130))
    )


def add_table_header_(page: Page, fonts: dict, pointer: dict) -> None:
    table: FixedColumnWidthTable = FixedColumnWidthTable(
        number_of_rows=1,
        number_of_columns=10,
        column_widths=[
            Decimal(20),
            Decimal(130),
            Decimal(65),
            Decimal(50),
            Decimal(30),
            Decimal(30),
            Decimal(45),
            Decimal(40),
            Decimal(30),
            Decimal(70),
        ],
    )

    attributes = [
        "№",
        "Наименование",
        "Произво-ль/ Артикул",
        "Цена за ед. с учетом НДС.",
        "ед изм",
        "К-во",
        "Сумма с учетом НДС.",
        "Кратность упаковки",
        "Наличие",
        "Срок поставки",
    ]

    for attribute in attributes:
        table.add(
            Paragraph(
                attribute,
                font=fonts["bold_font"],
                font_size=Decimal(6),
                horizontal_alignment=Alignment.CENTERED,
                vertical_alignment=Alignment.MIDDLE,
            )
        )

    table.set_padding_on_all_cells(
        Decimal(5), Decimal(1), Decimal(3), Decimal(1)
    )
    table.set_border_width_on_all_cells(Decimal(1.3))
    smallest_box = table.get_smallest_landscape_box()
    pointer["lower_y"] -= smallest_box.height

    table.paint(
        page,
        Rectangle(
            Decimal(pointer["lower_x"]),
            Decimal(pointer["lower_y"]),
            Decimal(510),
            Decimal(smallest_box.height),
        ),
    )


def build_table_(
    page: Page, products: pd.DataFrame, fonts: dict, pointer: dict
) -> pd.DataFrame:
    # print("in build_table_: " +  str(len(products)))

    while len(products) != 0:

        # print(products.iloc[0])
        print(f"added: {products['sku'].iloc[0]}")

        table: FixedColumnWidthTable = FixedColumnWidthTable(
            number_of_rows=1,
            number_of_columns=10,
            column_widths=[
                Decimal(20),
                Decimal(130),
                Decimal(65),
                Decimal(50),
                Decimal(30),
                Decimal(30),
                Decimal(45),
                Decimal(40),
                Decimal(30),
                Decimal(70),
            ],
        )
        table.add(
            Paragraph(
                str(products["index"].iloc[0]),
                font=fonts["regular_font"],
                font_size=Decimal(6),
                horizontal_alignment=Alignment.CENTERED,
                vertical_alignment=Alignment.MIDDLE,
            )
        )
        table.add(
            Paragraph(
                products["name"].iloc[0],
                font=fonts["regular_font"],
                font_size=Decimal(6),
            )
        )
        table.add(
            Paragraph(
                products["sku"].iloc[0],
                font=fonts["regular_font"],
                font_size=Decimal(6),
                horizontal_alignment=Alignment.CENTERED,
                vertical_alignment=Alignment.MIDDLE,
            )
        )
        table.add(
            Paragraph(
                products["str_cost"].iloc[0],
                font=fonts["regular_font"],
                font_size=Decimal(5),
                horizontal_alignment=Alignment.CENTERED,
                vertical_alignment=Alignment.MIDDLE,
            )
        )
        table.add(
            Paragraph(
                products["measure_unit"].iloc[0],
                font=fonts["regular_font"],
                font_size=Decimal(6),
                horizontal_alignment=Alignment.CENTERED,
                vertical_alignment=Alignment.MIDDLE,
            )
        )
        table.add(
            Paragraph(
                products["quantity"].iloc[0],
                font=fonts["regular_font"],
                font_size=Decimal(6),
                horizontal_alignment=Alignment.CENTERED,
                vertical_alignment=Alignment.MIDDLE,
            )
        )
        table.add(
            Paragraph(
                products["str_sum_cost"].iloc[0],
                font=fonts["regular_font"],
                font_size=Decimal(5),
                horizontal_alignment=Alignment.CENTERED,
                vertical_alignment=Alignment.MIDDLE,
            )
        )
        table.add(
            Paragraph(
                str("#Н/Д"),
                font=fonts["regular_font"],
                font_size=Decimal(6),
                horizontal_alignment=Alignment.CENTERED,
                vertical_alignment=Alignment.MIDDLE,
            )
        )
        table.add(
            Paragraph(
                products["quantity_in_stock"].iloc[0],
                font=fonts["regular_font"],
                font_size=Decimal(6),
                horizontal_alignment=Alignment.CENTERED,
                vertical_alignment=Alignment.MIDDLE,
            )
        )
        table.add(
            Paragraph(
                str("10-15 раб дней"),
                font=fonts["regular_font"],
                font_size=Decimal(6),
                horizontal_alignment=Alignment.RIGHT,
                vertical_alignment=Alignment.MIDDLE,
            )
        )
        table.set_padding_on_all_cells(
            Decimal(1), Decimal(0.8), Decimal(0), Decimal(0.8)
        )
        table.set_border_width_on_all_cells(Decimal(0.6))

        smallest_box = table.get_smallest_landscape_box()

        box = table.get_layout_box(
            Rectangle(
                Decimal(20),
                Decimal(0),
                Decimal(510),
                Decimal(smallest_box.height),
            )
        )

        if pointer["lower_y"] - box.height < 50:
            return products
        else:
            products.drop(products.index[0], inplace=True)
            pointer["lower_y"] -= box.height
            table.paint(
                page,
                Rectangle(
                    Decimal(pointer["lower_x"]),
                    Decimal(pointer["lower_y"]),
                    Decimal(510),
                    Decimal(box.height),
                ),
            )

    return products


exchange_rates_service = ExchangeRatesService()

ex_rates = exchange_rates_service.get_rates()

exchange_rates = {
    "EUR": round(ex_rates.eur_in_kzt, 2),
    "RUB": round(ex_rates.rub_in_kzt, 2),
}


def get_final_price(products: pd.DataFrame) -> str:
    final_price = products["sum_cost"].sum()

    if products.loc[0, "currency"] == "EUR":
        final_price = int(final_price * exchange_rates["EUR"])

    return str(int(final_price)) + " KZT"


def add_table_post_info_(
    page: Page, fonts: dict, final_price: str, pointer: dict
) -> None:
    table1: FixedColumnWidthTable = FixedColumnWidthTable(
        number_of_rows=1, number_of_columns=1, column_widths=[Decimal(200)]
    ).add(
        Paragraph(
            str("ВСЕГО СТОИМОСТЬ ОБОРУДОВАНИЯ С УЧЕТОМ НДС:"),
            font=fonts["bold_font"],
            font_size=Decimal(6),
            horizontal_alignment=Alignment.CENTERED,
            vertical_alignment=Alignment.BOTTOM,
        )
    )
    table1.set_border_width_on_all_cells(Decimal(1.2))
    pointer["lower_y"] -= 30
    table1.paint(
        page,
        Rectangle(
            Decimal(pointer["lower_x"]),
            Decimal(pointer["lower_y"]),
            Decimal(200),
            Decimal(12),
        ),
    )
    table2 = FixedColumnWidthTable(
        number_of_rows=1, number_of_columns=1, column_widths=[Decimal(150)]
    ).add(
        Paragraph(
            final_price,
            font=fonts["bold_font"],
            font_size=Decimal(6),
            horizontal_alignment=Alignment.CENTERED,
            vertical_alignment=Alignment.BOTTOM,
        )
    )
    table2.set_border_width_on_all_cells(Decimal(0.6))
    table2.paint(
        page,
        Rectangle(
            Decimal(340),
            Decimal(pointer["lower_y"]),
            Decimal(150),
            Decimal(12),
        ),
    )

    pointer["lower_y"] -= 30

    pdf_delimiter_path = pathlib.Path("static/pdf_delimiter.png")
    pdf_delimiter = Image(
        pdf_delimiter_path, width=Decimal(500), height=Decimal(20)
    )

    pdf_delimiter.paint(
        page,
        Rectangle(
            Decimal(pointer["lower_x"]),
            Decimal(pointer["lower_y"]),
            Decimal(500),
            Decimal(20),
        ),
    )

    pointer["lower_y"] -= 120

    post_text = """Условия оплаты: 100% Предоплата
    Условие поставки: DDP г.Астана.
    Срок действия предложения: 5-7 календарных дней.
    Срок поставки оборудования: исчисляется с момента поступление предоплаты
    Контактное лицо поставщика: Калмаханов А.Г.\n \n
    E-mail: asb-top@mail.ru
    Моб: +7 708 425 62 82
    Тел: +7 7172 25-62-82
    Е. www.asb-top.kz \n \n
    С уважением,
    Калмаханов А.Г."""
    post_paragraph = Paragraph(
        text=post_text,
        font=fonts["semibold_font"],
        font_size=Decimal(6),
        respect_newlines_in_text=True,
    )
    post_paragraph.paint(
        page,
        Rectangle(
            Decimal(pointer["lower_x"] + 30),
            Decimal(pointer["lower_y"]),
            Decimal(200),
            Decimal(120),
        ),
    )
