import os
import json
import sys
import os

parent_dir = os.path.abspath(os.path.join(os.path.dirname(__file__), ".."))
sys.path.insert(0, parent_dir)

import pytest
from unittest.mock import mock_open, patch
from API.products_service import ProductsService


# mock of product data
@pytest.fixture
def mock_product_data():
    return [
        {
            "code": "7007063",
            "sku": "RBV 620 F FS",
            "name": "Секция угловая 90° внешняя, h=60 мм, B=200 мм, FS",
            "description": "90° vertical bend",
            "quantity_in_stock": 23,
            "cost": 42.199542,
            "unit": "ШТ",
        },
        {
            "code": "6043102",
            "sku": "RBL 90 660 FS RU",
            "name": "Секция угловая 90°, h=60 мм, B=600 мм, FS",
            "description": "90° bend",
            "quantity_in_stock": 35,
            "cost": 48.90186,
            "unit": "ШТ",
        },
        {
            "code": "6067091",
            "sku": "RLVK 60 FS RU",
            "name": "Соединитель продольный RLVK 60 FS для лотка LKS 60 мм",
            "description": "Straight connector",
            "quantity_in_stock": 3850,
            "cost": 1.67076,
            "unit": "ШТ",
        },
    ]


def test_get_product_info(mock_product_data):
    sku_list = ["7007063", "9999"]  # '9999' does not exists on website
    expected_output = [
        {
            "sku": "7007063",
            "measure_unit": "ШТ",
            "cost": 42.2,
            "quantity_in_stock": 23,
        }
    ]

    with patch.dict(os.environ, {"PRODUCT_QUANTITY_PATH": "dummy_path"}):
        with patch(
            "builtins.open", mock_open(read_data=json.dumps(mock_product_data))
        ):
            product_manager = ProductsService()
            result = product_manager.get_product_info(sku_list)

    assert result == expected_output


def test_get_product_info_no_sku_found(mock_product_data):
    sku_list = ["999", "444"]  # SKU, которых нет в данных
    expected_output = []

    with patch.dict(os.environ, {"PRODUCT_QUANTITY_PATH": "dummy_path"}):
        with patch(
            "builtins.open", mock_open(read_data=json.dumps(mock_product_data))
        ):
            product_manager = ProductsService()
            result = product_manager.get_product_info(sku_list)

    assert result == expected_output


def test_get_product_info_with_invalid_cost(mock_product_data):
    mock_product_data.append(
        {
            "code": "7007013",
            "sku": "RBV 620 S FS",
            "name": "Секция угловая 90° внутренняя, h=60 мм, B=200 мм, FS",
            "description": "90° vertical bend",
            "quantity_in_stock": 25,
            "cost": None,
            "unit": "ШТ",
        },
    )

    sku_list = ["7007063", "7007013"]
    expected_output = [
        {
            "sku": "7007063",
            "measure_unit": "ШТ",
            "cost": 42.2,
            "quantity_in_stock": 23,
        },
        {
            "sku": "7007013",
            "measure_unit": "ШТ",
            "cost": None,
            "quantity_in_stock": 25,
        },
    ]

    with patch.dict(os.environ, {"PRODUCT_QUANTITY_PATH": "dummy_path"}):
        with patch(
            "builtins.open", mock_open(read_data=json.dumps(mock_product_data))
        ):
            product_manager = ProductsService()
            result = product_manager.get_product_info(sku_list)

    assert result == expected_output


"""
{
"customer_info" : {"phone": "89973377777", "name": "aaaa"},
"products": [
    {
        "sku": "2007045",
        "quantity": "3"
    },
    {
        "sku": "2363010",
        "quantity": "4"
    },
    {
        "sku": "7007063",
        "quantity": "11"
    },
    {
        "sku": "5314666",
        "quantity": "5"
    }

]}

"""


class MockExRates:
    def __init__(self):
        self.rub_in_kzt = 5.123
        self.eur_in_kzt = 506.323


exchange_rates_service = MockExRates()


@pytest.fixture
def setup_data():
    satu_info = [
        {
            "sku": "2363010",
            "currency": "EUR",
            "name": "Холодная заливочная масса Aquasit, картридж",
        },
        {
            "sku": "2007045",
            "currency": "EUR",
            "name": "Распределительная коробка T40, влагозащита, IP 55, 90x90x52 мм, с герметичными вводами 7 x M25",
        },
        {
            "sku": "5314666",
            "currency": "EUR",
            "name": "Крестовой соединитель для плоских проводников до 40 мм",
        },
        {
            "sku": "7007063",
            "currency": "EUR",
            "name": "Вертикальный угол 90° внешний 60x200 мм",
        },
    ]

    files_info = [
        {
            "sku": "2007045",
            "measure_unit": "ШТ",
            "cost": 1.71,
            "quantity_in_stock": "1448",
        },
        {
            "sku": "2363010",
            "measure_unit": "ШТ",
            "cost": 38.11,
            "quantity_in_stock": "1",
        },
        {
            "sku": "7007063",
            "measure_unit": "ШТ",
            "cost": 42.2,
            "quantity_in_stock": "23",
        },
        {
            "sku": "5314666",
            "measure_unit": "ШТ",
            "cost": 6.93,
            "quantity_in_stock": "445",
        },
    ]

    products = [
        ("2007045", 3),
        ("2363010", 4),
        ("7007063", 11),
        ("5314666", 5),
    ]

    return satu_info, files_info, products, MockExRates()


def test_build_co_dataframe(setup_data):
    a = exchange_rates_service
    df1, df2, df3, ex_rates = setup_data

    # Создайте экземпляр вашего класса
    obj = ProductsService()

    # Вызовите метод
    commercial_offer_df = obj.build_co_dataframe(df1, df2, df3, ex_rates)

    # Проверьте количество строк в итоговом DataFrame
    assert (
        len(commercial_offer_df) == 4
    )  # Ожидаемое количество строк после объединения

    # Проверьте корректность расчета строки str_cost
    expected_str_costs = [
        "865.81 KZT",
        "19295.86 KZT",
        "3508.8 KZT",
        "21366.7 KZT",
    ]  # Пример ожидаемых цен
    assert list(commercial_offer_df["str_cost"]) == expected_str_costs

    # Проверьте корректность расчета sum_cost
    expected_sum_costs = [
        5.13,
        152.44,
        34.65,
        464.20000000000005,
    ]  # Пример ожидаемых значений
    assert list(commercial_offer_df["sum_cost"]) == expected_sum_costs

    # Проверьте корректность индексации
    assert (
        commercial_offer_df["index"] == [1, 2, 3, 4]
    ).all()  # Проверка индекса


if __name__ == "__main__":
    pytest.main()
